-module(pevensie@postgres).
-compile([no_auto_import, nowarn_unused_vars, nowarn_unused_function, nowarn_nomatch]).

-export([default_config/0, user_decoder/1, session_decoder/0, new_cache_driver/1, main/0, new_auth_driver/1]).
-export_type([ip_version/0, postgres_config/0, postgres/0, postgres_error/0]).

-type ip_version() :: ipv4 | ipv6.

-type postgres_config() :: {postgres_config,
        binary(),
        integer(),
        binary(),
        binary(),
        gleam@option:option(binary()),
        boolean(),
        list({binary(), binary()}),
        integer(),
        integer(),
        integer(),
        integer(),
        boolean(),
        ip_version(),
        integer()}.

-opaque postgres() :: {postgres,
        postgres_config(),
        gleam@option:option(pog:connection())}.

-type postgres_error() :: {constraint_violated, binary(), binary(), binary()} |
    {postgresql_error, binary(), binary(), binary()} |
    connection_unavailable.

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 113).
-spec default_config() -> postgres_config().
default_config() ->
    {postgres_config,
        <<"127.0.0.1"/utf8>>,
        5432,
        <<"postgres"/utf8>>,
        <<"postgres"/utf8>>,
        none,
        false,
        [],
        1,
        50,
        1000,
        1000,
        false,
        ipv4,
        5000}.

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 132).
-spec postgres_config_to_pog_config(postgres_config()) -> pog:config().
postgres_config_to_pog_config(Config) ->
    {config,
        erlang:element(2, Config),
        erlang:element(3, Config),
        erlang:element(4, Config),
        erlang:element(5, Config),
        erlang:element(6, Config),
        case erlang:element(7, Config) of
            true ->
                ssl_verified;

            false ->
                ssl_disabled
        end,
        erlang:element(8, Config),
        erlang:element(9, Config),
        erlang:element(10, Config),
        erlang:element(11, Config),
        erlang:element(12, Config),
        erlang:element(13, Config),
        case erlang:element(14, Config) of
            ipv4 ->
                ipv4;

            ipv6 ->
                ipv6
        end,
        false,
        erlang:element(15, Config)}.

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 158).
-spec pog_query_error_to_postgres_error(pog:query_error()) -> postgres_error().
pog_query_error_to_postgres_error(Err) ->
    case Err of
        {postgresql_error, Code, Name, Message} ->
            {postgresql_error, Code, Name, Message};

        connection_unavailable ->
            connection_unavailable;

        {constraint_violated, Message@1, Constraint, Detail} ->
            {constraint_violated, Message@1, Constraint, Detail};

        _ ->
            erlang:error(#{gleam_error => panic,
                    message => <<"pog Unexpected* error - should not occur if queries are written correctly"/utf8>>,
                    module => <<"pevensie/postgres"/utf8>>,
                    function => <<"pog_query_error_to_postgres_error"/utf8>>,
                    line => 166})
    end.

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 170).
-spec pog_query_error_to_pevensie_error(
    pog:query_error(),
    fun((postgres_error()) -> QDO)
) -> QDO.
pog_query_error_to_pevensie_error(Err, Pevensie_error) ->
    _pipe = Err,
    _pipe@1 = pog_query_error_to_postgres_error(_pipe),
    Pevensie_error(_pipe@1).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 179).
-spec tempo_datetime_to_pog_timestamp(tempo:date_time()) -> pog:timestamp().
tempo_datetime_to_pog_timestamp(Datetime) ->
    Date = tempo@datetime:get_date(Datetime),
    Time = tempo@datetime:get_time(Datetime),
    {timestamp,
        {date,
            tempo@date:get_year(Date),
            begin
                _pipe = tempo@date:get_month(Date),
                tempo@month:to_int(_pipe)
            end,
            tempo@date:get_day(Date)},
        {time,
            tempo@time:get_hour(Time),
            tempo@time:get_minute(Time),
            tempo@time:get_second(Time),
            tempo@time:get_nanosecond(Time) div 1000}}.

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 245).
-spec connect(postgres()) -> {ok, postgres()} |
    {error, pevensie@drivers:connect_error(postgres_error())}.
connect(Driver) ->
    case Driver of
        {postgres, Config, none} ->
            Conn = begin
                _pipe = Config,
                _pipe@1 = postgres_config_to_pog_config(_pipe),
                pog_ffi:connect(_pipe@1)
            end,
            {ok, {postgres, Config, {some, Conn}}};

        {postgres, _, {some, _}} ->
            {error, already_connected}
    end.

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 262).
-spec disconnect(postgres()) -> {ok, postgres()} |
    {error, pevensie@drivers:disconnect_error(postgres_error())}.
disconnect(Driver) ->
    case Driver of
        {postgres, Config, {some, Conn}} ->
            _ = pog_ffi:disconnect(Conn),
            {ok, {postgres, Config, none}};

        {postgres, _, none} ->
            {error, not_connected}
    end.

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 293).
-spec db_app_metadata_decoder() -> gleam@dynamic@decode:decoder(pevensie@auth:app_metadata()).
db_app_metadata_decoder() ->
    gleam@dynamic@decode:then(
        {decoder, fun gleam@dynamic@decode:decode_string/1},
        fun(Data_string) ->
            case gleam@json:parse(
                Data_string,
                pevensie@auth:app_metadata_decoder()
            ) of
                {ok, Data} ->
                    gleam@dynamic@decode:success(Data);

                {error, _} ->
                    gleam@dynamic@decode:failure(
                        {app_metadata, maps:new()},
                        <<"AppMetadata"/utf8>>
                    )
            end
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 307).
-spec db_user_metadata_decoder(gleam@dynamic@decode:decoder(QEB)) -> gleam@dynamic@decode:decoder(QEB).
db_user_metadata_decoder(User_metadata_decoder) ->
    Zero_value = pevensie_postgres_ffi:unsafe_coerce(nil),
    gleam@dynamic@decode:then(
        {decoder, fun gleam@dynamic@decode:decode_string/1},
        fun(Data_string) ->
            case gleam@json:parse(Data_string, User_metadata_decoder) of
                {ok, Data} ->
                    gleam@dynamic@decode:success(Data);

                {error, _} ->
                    gleam@dynamic@decode:failure(
                        Zero_value,
                        <<"UserMetadata"/utf8>>
                    )
            end
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 320).
-spec user_decoder(gleam@dynamic@decode:decoder(QEE)) -> gleam@dynamic@decode:decoder(pevensie@auth:user(QEE)).
user_decoder(User_metadata_decoder) ->
    gleam@dynamic@decode:field(
        0,
        {decoder, fun gleam@dynamic@decode:decode_string/1},
        fun(Id) ->
            gleam@dynamic@decode:field(
                1,
                {decoder, fun gleam@dynamic@decode:decode_int/1},
                fun(Created_at_micro) ->
                    gleam@dynamic@decode:field(
                        2,
                        {decoder, fun gleam@dynamic@decode:decode_int/1},
                        fun(Updated_at_micro) ->
                            gleam@dynamic@decode:field(
                                3,
                                gleam@dynamic@decode:optional(
                                    {decoder,
                                        fun gleam@dynamic@decode:decode_int/1}
                                ),
                                fun(Deleted_at_micro) ->
                                    gleam@dynamic@decode:field(
                                        4,
                                        gleam@dynamic@decode:optional(
                                            {decoder,
                                                fun gleam@dynamic@decode:decode_string/1}
                                        ),
                                        fun(Role) ->
                                            gleam@dynamic@decode:field(
                                                5,
                                                {decoder,
                                                    fun gleam@dynamic@decode:decode_string/1},
                                                fun(Email) ->
                                                    gleam@dynamic@decode:field(
                                                        6,
                                                        gleam@dynamic@decode:optional(
                                                            {decoder,
                                                                fun gleam@dynamic@decode:decode_string/1}
                                                        ),
                                                        fun(Password_hash) ->
                                                            gleam@dynamic@decode:field(
                                                                7,
                                                                gleam@dynamic@decode:optional(
                                                                    {decoder,
                                                                        fun gleam@dynamic@decode:decode_int/1}
                                                                ),
                                                                fun(
                                                                    Email_confirmed_at_micro
                                                                ) ->
                                                                    gleam@dynamic@decode:field(
                                                                        8,
                                                                        gleam@dynamic@decode:optional(
                                                                            {decoder,
                                                                                fun gleam@dynamic@decode:decode_string/1}
                                                                        ),
                                                                        fun(
                                                                            Phone_number
                                                                        ) ->
                                                                            gleam@dynamic@decode:field(
                                                                                9,
                                                                                gleam@dynamic@decode:optional(
                                                                                    {decoder,
                                                                                        fun gleam@dynamic@decode:decode_int/1}
                                                                                ),
                                                                                fun(
                                                                                    Phone_number_confirmed_at_micro
                                                                                ) ->
                                                                                    gleam@dynamic@decode:field(
                                                                                        10,
                                                                                        gleam@dynamic@decode:optional(
                                                                                            {decoder,
                                                                                                fun gleam@dynamic@decode:decode_int/1}
                                                                                        ),
                                                                                        fun(
                                                                                            Last_sign_in_micro
                                                                                        ) ->
                                                                                            gleam@dynamic@decode:field(
                                                                                                11,
                                                                                                db_app_metadata_decoder(
                                                                                                    
                                                                                                ),
                                                                                                fun(
                                                                                                    App_metadata
                                                                                                ) ->
                                                                                                    gleam@dynamic@decode:field(
                                                                                                        12,
                                                                                                        db_user_metadata_decoder(
                                                                                                            User_metadata_decoder
                                                                                                        ),
                                                                                                        fun(
                                                                                                            User_metadata
                                                                                                        ) ->
                                                                                                            gleam@dynamic@decode:field(
                                                                                                                13,
                                                                                                                gleam@dynamic@decode:optional(
                                                                                                                    {decoder,
                                                                                                                        fun gleam@dynamic@decode:decode_int/1}
                                                                                                                ),
                                                                                                                fun(
                                                                                                                    Banned_until_micro
                                                                                                                ) ->
                                                                                                                    Created_at = tempo@datetime:from_unix_micro_utc(
                                                                                                                        Created_at_micro
                                                                                                                    ),
                                                                                                                    Updated_at = tempo@datetime:from_unix_micro_utc(
                                                                                                                        Updated_at_micro
                                                                                                                    ),
                                                                                                                    Deleted_at = gleam@option:map(
                                                                                                                        Deleted_at_micro,
                                                                                                                        fun tempo@datetime:from_unix_micro_utc/1
                                                                                                                    ),
                                                                                                                    Email_confirmed_at = gleam@option:map(
                                                                                                                        Email_confirmed_at_micro,
                                                                                                                        fun tempo@datetime:from_unix_micro_utc/1
                                                                                                                    ),
                                                                                                                    Phone_number_confirmed_at = gleam@option:map(
                                                                                                                        Phone_number_confirmed_at_micro,
                                                                                                                        fun tempo@datetime:from_unix_micro_utc/1
                                                                                                                    ),
                                                                                                                    Last_sign_in = gleam@option:map(
                                                                                                                        Last_sign_in_micro,
                                                                                                                        fun tempo@datetime:from_unix_micro_utc/1
                                                                                                                    ),
                                                                                                                    Banned_until = gleam@option:map(
                                                                                                                        Banned_until_micro,
                                                                                                                        fun tempo@datetime:from_unix_micro_utc/1
                                                                                                                    ),
                                                                                                                    gleam@dynamic@decode:success(
                                                                                                                        {user,
                                                                                                                            Id,
                                                                                                                            Created_at,
                                                                                                                            Updated_at,
                                                                                                                            Deleted_at,
                                                                                                                            Role,
                                                                                                                            Email,
                                                                                                                            Password_hash,
                                                                                                                            Email_confirmed_at,
                                                                                                                            Phone_number,
                                                                                                                            Phone_number_confirmed_at,
                                                                                                                            Last_sign_in,
                                                                                                                            App_metadata,
                                                                                                                            User_metadata,
                                                                                                                            Banned_until}
                                                                                                                    )
                                                                                                                end
                                                                                                            )
                                                                                                        end
                                                                                                    )
                                                                                                end
                                                                                            )
                                                                                        end
                                                                                    )
                                                                                end
                                                                            )
                                                                        end
                                                                    )
                                                                end
                                                            )
                                                        end
                                                    )
                                                end
                                            )
                                        end
                                    )
                                end
                            )
                        end
                    )
                end
            )
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 494).
-spec update_field_to_sql(
    pevensie@auth:update_field(QEX),
    fun((QEX) -> pog:value())
) -> pevensie@auth:update_field(pog:value()).
update_field_to_sql(Field, Sql_type) ->
    case Field of
        {set, Value} ->
            {set, Sql_type(Value)};

        ignore ->
            ignore
    end.

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 668).
-spec session_decoder() -> gleam@dynamic@decode:decoder(pevensie@auth:session()).
session_decoder() ->
    gleam@dynamic@decode:field(
        0,
        {decoder, fun gleam@dynamic@decode:decode_string/1},
        fun(Id) ->
            gleam@dynamic@decode:field(
                1,
                {decoder, fun gleam@dynamic@decode:decode_string/1},
                fun(User_id) ->
                    gleam@dynamic@decode:field(
                        2,
                        {decoder, fun gleam@dynamic@decode:decode_int/1},
                        fun(Created_at_micro) ->
                            gleam@dynamic@decode:field(
                                3,
                                gleam@dynamic@decode:optional(
                                    {decoder,
                                        fun gleam@dynamic@decode:decode_int/1}
                                ),
                                fun(Expires_at_micro) ->
                                    gleam@dynamic@decode:field(
                                        4,
                                        gleam@dynamic@decode:optional(
                                            pevensie@net:ip_address_decoder()
                                        ),
                                        fun(Ip) ->
                                            gleam@dynamic@decode:field(
                                                5,
                                                gleam@dynamic@decode:optional(
                                                    {decoder,
                                                        fun gleam@dynamic@decode:decode_string/1}
                                                ),
                                                fun(User_agent) ->
                                                    Created_at = tempo@datetime:from_unix_micro_utc(
                                                        Created_at_micro
                                                    ),
                                                    Expires_at = gleam@option:map(
                                                        Expires_at_micro,
                                                        fun tempo@datetime:from_unix_micro_utc/1
                                                    ),
                                                    gleam@dynamic@decode:success(
                                                        {session,
                                                            Id,
                                                            Created_at,
                                                            Expires_at,
                                                            User_id,
                                                            Ip,
                                                            User_agent}
                                                    )
                                                end
                                            )
                                        end
                                    )
                                end
                            )
                        end
                    )
                end
            )
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 864).
-spec delete_session(postgres(), binary()) -> {ok, nil} |
    {error, pevensie@auth:delete_error(postgres_error())}.
delete_session(Driver, Session_id) ->
    {postgres, _, {some, Conn}} = case Driver of
        {postgres, _, {some, _}} -> Driver;
        _assert_fail ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"delete_session"/utf8>>,
                        line => 868})
    end,
    Sql = <<"
    delete from pevensie.\"session\"
    where id = $1
    returning id
      "/utf8>>,
    _pipe = pog:'query'(Sql),
    _pipe@1 = pog:parameter(_pipe, pog_ffi:coerce(Session_id)),
    _pipe@2 = pog:execute(_pipe@1, Conn),
    _pipe@3 = gleam@result:replace(_pipe@2, nil),
    gleam@result:map_error(
        _pipe@3,
        fun(_capture) ->
            pog_query_error_to_pevensie_error(
                _capture,
                fun(Field@0) -> {delete_driver_error, Field@0} end
            )
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 887).
-spec one_time_token_type_to_prefix(pevensie@auth:one_time_token_type()) -> binary().
one_time_token_type_to_prefix(Token_type) ->
    case Token_type of
        password_reset ->
            <<"pr"/utf8>>
    end.

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 893).
-spec one_time_token_type_to_pg_enum(pevensie@auth:one_time_token_type()) -> binary().
one_time_token_type_to_pg_enum(Token_type) ->
    case Token_type of
        password_reset ->
            <<"password-reset"/utf8>>
    end.

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 899).
-spec create_one_time_token(
    postgres(),
    binary(),
    pevensie@auth:one_time_token_type(),
    integer()
) -> {ok, binary()} | {error, pevensie@auth:create_error(postgres_error())}.
create_one_time_token(Driver, User_id, Token_type, Ttl_seconds) ->
    {postgres, _, {some, Conn}} = case Driver of
        {postgres, _, {some, _}} -> Driver;
        _assert_fail ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"create_one_time_token"/utf8>>,
                        line => 905})
    end,
    Token = <<(one_time_token_type_to_prefix(Token_type))/binary,
        (begin
            _pipe = crypto:strong_rand_bytes(36),
            gleam_stdlib:bit_array_base64_encode(_pipe, true)
        end)/binary>>,
    gleam@result:'try'(
        begin
            _pipe@1 = pevensie@internal@encode:sha256_hash(Token, Token),
            gleam@result:replace_error(
                _pipe@1,
                {create_internal_error, <<"Failed to hash token"/utf8>>}
            )
        end,
        fun(Token_hash) ->
            Sql = <<"
  insert into pevensie.one_time_token (user_id, token_type, token_hash, expires_at)
  values ($1, $2, $3, now() + interval '$4 seconds')
    "/utf8>>,
            _pipe@2 = pog:'query'(Sql),
            _pipe@3 = pog:parameter(_pipe@2, pog_ffi:coerce(User_id)),
            _pipe@4 = pog:parameter(
                _pipe@3,
                pog_ffi:coerce(one_time_token_type_to_pg_enum(Token_type))
            ),
            _pipe@5 = pog:parameter(_pipe@4, pog_ffi:coerce(Token_hash)),
            _pipe@6 = pog:parameter(
                _pipe@5,
                pog_ffi:coerce(erlang:integer_to_binary(Ttl_seconds))
            ),
            _pipe@7 = pog:execute(_pipe@6, Conn),
            _pipe@8 = gleam@result:replace(_pipe@7, Token),
            gleam@result:map_error(
                _pipe@8,
                fun(_capture) ->
                    pog_query_error_to_pevensie_error(
                        _capture,
                        fun(Field@0) -> {create_driver_error, Field@0} end
                    )
                end
            )
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 934).
-spec validate_one_time_token(
    postgres(),
    binary(),
    pevensie@auth:one_time_token_type(),
    binary()
) -> {ok, nil} | {error, pevensie@auth:get_error(postgres_error())}.
validate_one_time_token(Driver, User_id, Token_type, Token) ->
    {postgres, _, {some, Conn}} = case Driver of
        {postgres, _, {some, _}} -> Driver;
        _assert_fail ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"validate_one_time_token"/utf8>>,
                        line => 940})
    end,
    gleam@result:'try'(
        begin
            _pipe = pevensie@internal@encode:sha256_hash(Token, Token),
            gleam@result:replace_error(
                _pipe,
                {get_internal_error, <<"Failed to hash token"/utf8>>}
            )
        end,
        fun(Token_hash) ->
            Sql = <<"
  select id from pevensie.one_time_token
  where user_id = $1
    and token_type = $2
    and token_hash = $3
    and deleted_at is null
    and used_at is null
    and expires_at < now() -- USE THIS TO CREATE AN ERROR LATER?
    "/utf8>>,
            _pipe@1 = pog:'query'(Sql),
            _pipe@2 = pog:parameter(_pipe@1, pog_ffi:coerce(User_id)),
            _pipe@3 = pog:parameter(
                _pipe@2,
                pog_ffi:coerce(one_time_token_type_to_pg_enum(Token_type))
            ),
            _pipe@4 = pog:parameter(_pipe@3, pog_ffi:coerce(Token_hash)),
            _pipe@5 = pog:execute(_pipe@4, Conn),
            _pipe@6 = gleam@result:replace(_pipe@5, nil),
            gleam@result:map_error(
                _pipe@6,
                fun(_capture) ->
                    pog_query_error_to_pevensie_error(
                        _capture,
                        fun(Field@0) -> {get_driver_error, Field@0} end
                    )
                end
            )
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 967).
-spec use_one_time_token(
    postgres(),
    binary(),
    pevensie@auth:one_time_token_type(),
    binary()
) -> {ok, nil} | {error, pevensie@auth:update_error(postgres_error())}.
use_one_time_token(Driver, User_id, Token_type, Token) ->
    {postgres, _, {some, Conn}} = case Driver of
        {postgres, _, {some, _}} -> Driver;
        _assert_fail ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"use_one_time_token"/utf8>>,
                        line => 973})
    end,
    gleam@result:'try'(
        begin
            _pipe = pevensie@internal@encode:sha256_hash(Token, Token),
            gleam@result:replace_error(
                _pipe,
                {update_internal_error, <<"Failed to hash token"/utf8>>}
            )
        end,
        fun(Token_hash) ->
            Sql = <<"
  update from pevensie.one_time_token
  set used_at = now()
  where user_id = $1
    and token_type = $2
    and token_hash = $3
    and deleted_at is null
    and used_at is null
    and expires_at < now() -- USE THIS TO CREATE AN ERROR LATER?
  returning id
    "/utf8>>,
            _pipe@1 = pog:'query'(Sql),
            _pipe@2 = pog:parameter(_pipe@1, pog_ffi:coerce(User_id)),
            _pipe@3 = pog:parameter(
                _pipe@2,
                pog_ffi:coerce(one_time_token_type_to_pg_enum(Token_type))
            ),
            _pipe@4 = pog:parameter(_pipe@3, pog_ffi:coerce(Token_hash)),
            _pipe@5 = pog:execute(_pipe@4, Conn),
            _pipe@6 = gleam@result:replace(_pipe@5, nil),
            gleam@result:map_error(
                _pipe@6,
                fun(_capture) ->
                    pog_query_error_to_pevensie_error(
                        _capture,
                        fun(Field@0) -> {update_driver_error, Field@0} end
                    )
                end
            )
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1005).
-spec delete_one_time_token(
    postgres(),
    binary(),
    pevensie@auth:one_time_token_type(),
    binary()
) -> {ok, nil} | {error, pevensie@auth:delete_error(postgres_error())}.
delete_one_time_token(Driver, User_id, Token_type, Token) ->
    {postgres, _, {some, Conn}} = case Driver of
        {postgres, _, {some, _}} -> Driver;
        _assert_fail ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"delete_one_time_token"/utf8>>,
                        line => 1011})
    end,
    gleam@result:'try'(
        begin
            _pipe = pevensie@internal@encode:sha256_hash(Token, Token),
            gleam@result:replace_error(
                _pipe,
                {delete_internal_error, <<"Failed to hash token"/utf8>>}
            )
        end,
        fun(Token_hash) ->
            Sql = <<"
  update from pevensie.one_time_token
  set deleted_at = now()
  where user_id = $1
    and token_type = $2
    and token_hash = $3
  returning id
    "/utf8>>,
            _pipe@1 = pog:'query'(Sql),
            _pipe@2 = pog:parameter(_pipe@1, pog_ffi:coerce(User_id)),
            _pipe@3 = pog:parameter(
                _pipe@2,
                pog_ffi:coerce(one_time_token_type_to_pg_enum(Token_type))
            ),
            _pipe@4 = pog:parameter(_pipe@3, pog_ffi:coerce(Token_hash)),
            _pipe@5 = pog:execute(_pipe@4, Conn),
            _pipe@6 = gleam@result:replace(_pipe@5, nil),
            gleam@result:map_error(
                _pipe@6,
                fun(_capture) ->
                    pog_query_error_to_pevensie_error(
                        _capture,
                        fun(Field@0) -> {delete_driver_error, Field@0} end
                    )
                end
            )
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1073).
-spec set_in_cache(
    postgres(),
    binary(),
    binary(),
    binary(),
    gleam@option:option(integer())
) -> {ok, nil} | {error, pevensie@cache:set_error(postgres_error())}.
set_in_cache(Driver, Resource_type, Key, Value, Ttl_seconds) ->
    {postgres, _, {some, Conn}} = case Driver of
        {postgres, _, {some, _}} -> Driver;
        _assert_fail ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"set_in_cache"/utf8>>,
                        line => 1080})
    end,
    Expires_at_sql = case Ttl_seconds of
        none ->
            <<"null"/utf8>>;

        {some, Ttl_seconds@1} ->
            <<<<"now() + interval '"/utf8,
                    (erlang:integer_to_binary(Ttl_seconds@1))/binary>>/binary,
                " seconds'"/utf8>>
    end,
    Sql = <<<<"
    insert into pevensie.\"cache\" (
      resource_type,
      key,
      value,
      expires_at
    ) values (
      $1,
      $2,
      $3,
      "/utf8,
            Expires_at_sql/binary>>/binary,
        "
    )
    on conflict (resource_type, key) do update set value = EXCLUDED.value, expires_at = EXCLUDED.expires_at"/utf8>>,
    _pipe = pog:'query'(Sql),
    _pipe@1 = pog:parameter(_pipe, pog_ffi:coerce(Resource_type)),
    _pipe@2 = pog:parameter(_pipe@1, pog_ffi:coerce(Key)),
    _pipe@3 = pog:parameter(_pipe@2, pog_ffi:coerce(Value)),
    _pipe@4 = pog:execute(_pipe@3, Conn),
    _pipe@5 = gleam@result:replace(_pipe@4, nil),
    gleam@result:map_error(
        _pipe@5,
        fun(_capture) ->
            pog_query_error_to_pevensie_error(
                _capture,
                fun(Field@0) -> {set_driver_error, Field@0} end
            )
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1162).
-spec delete_from_cache(postgres(), binary(), binary()) -> {ok, nil} |
    {error, pevensie@cache:delete_error(postgres_error())}.
delete_from_cache(Driver, Resource_type, Key) ->
    {postgres, _, {some, Conn}} = case Driver of
        {postgres, _, {some, _}} -> Driver;
        _assert_fail ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"delete_from_cache"/utf8>>,
                        line => 1167})
    end,
    Sql = <<"
    delete from pevensie.\"cache\"
    where resource_type = $1 and key = $2"/utf8>>,
    _pipe = pog:'query'(Sql),
    _pipe@1 = pog:parameter(_pipe, pog_ffi:coerce(Resource_type)),
    _pipe@2 = pog:parameter(_pipe@1, pog_ffi:coerce(Key)),
    _pipe@3 = pog:execute(_pipe@2, Conn),
    _pipe@4 = gleam@result:replace(_pipe@3, nil),
    gleam@result:map_error(
        _pipe@4,
        fun(_capture) ->
            pog_query_error_to_pevensie_error(
                _capture,
                fun(Field@0) -> {delete_driver_error, Field@0} end
            )
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1110).
-spec get_from_cache(postgres(), binary(), binary()) -> {ok, binary()} |
    {error, pevensie@cache:get_error(postgres_error())}.
get_from_cache(Driver, Resource_type, Key) ->
    {postgres, _, {some, Conn}} = case Driver of
        {postgres, _, {some, _}} -> Driver;
        _assert_fail ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"get_from_cache"/utf8>>,
                        line => 1115})
    end,
    Sql = <<"
    select
      value::text,
      -- Returns true only if the exporation time is
      -- set and has passed
      (expires_at is not null and expires_at < now()) as expired
    from pevensie.\"cache\"
    where resource_type = $1 and key = $2"/utf8>>,
    Query_result = begin
        _pipe = pog:'query'(Sql),
        _pipe@1 = pog:parameter(_pipe, pog_ffi:coerce(Resource_type)),
        _pipe@2 = pog:parameter(_pipe@1, pog_ffi:coerce(Key)),
        _pipe@3 = pog:returning(
            _pipe@2,
            (gleam@dynamic@decode:field(
                0,
                {decoder, fun gleam@dynamic@decode:decode_string/1},
                fun(Value) ->
                    gleam@dynamic@decode:field(
                        1,
                        {decoder, fun gleam@dynamic@decode:decode_bool/1},
                        fun(Expired) ->
                            gleam@dynamic@decode:success({Value, Expired})
                        end
                    )
                end
            ))
        ),
        _pipe@4 = pog:execute(_pipe@3, Conn),
        gleam@result:map_error(
            _pipe@4,
            fun(_capture) ->
                pog_query_error_to_pevensie_error(
                    _capture,
                    fun(Field@0) -> {get_driver_error, Field@0} end
                )
            end
        )
    end,
    gleam@result:'try'(
        Query_result,
        fun(Response) -> case erlang:element(3, Response) of
                [{Value@1, false}] ->
                    {ok, Value@1};

                [{_, true}] ->
                    gleam@erlang@process:start(
                        fun() ->
                            delete_from_cache(Driver, Resource_type, Key)
                        end,
                        false
                    ),
                    {error, got_too_few_records};

                [] ->
                    {error, got_too_few_records};

                _ ->
                    {error, got_too_many_records}
            end end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1060).
-spec new_cache_driver(postgres_config()) -> pevensie@cache:cache_driver(postgres(), postgres_error()).
new_cache_driver(Config) ->
    {cache_driver,
        {postgres, Config, none},
        fun connect/1,
        fun disconnect/1,
        fun set_in_cache/5,
        fun get_from_cache/3,
        fun delete_from_cache/3}.

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1186).
-spec add_error_context(binary(), binary()) -> binary().
add_error_context(Error, Context) ->
    <<<<Context/binary, ": "/utf8>>/binary, Error/binary>>.

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1190).
-spec check_pevensie_schema_exists(pog:connection()) -> {ok, boolean()} |
    {error, binary()}.
check_pevensie_schema_exists(Tx) ->
    Query_result = begin
        _pipe = pog:'query'(
            <<"select schema_name from information_schema.schemata where schema_name = 'pevensie'"/utf8>>
        ),
        _pipe@1 = pog:returning(
            _pipe,
            (gleam@dynamic@decode:field(
                0,
                {decoder, fun gleam@dynamic@decode:decode_string/1},
                fun(Value) -> gleam@dynamic@decode:success(Value) end
            ))
        ),
        _pipe@2 = pog:execute(_pipe@1, Tx),
        gleam@result:map_error(_pipe@2, fun gleam@string:inspect/1)
    end,
    gleam@result:'try'(
        Query_result,
        fun(Response) -> case erlang:element(3, Response) of
                [_] ->
                    {ok, true};

                [] ->
                    {ok, false};

                _ ->
                    {error, <<"Too many rows"/utf8>>}
            end end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1210).
-spec get_module_version(pog:connection(), binary()) -> {ok,
        gleam@option:option(tempo:date_time())} |
    {error, binary()}.
get_module_version(Tx, Module) ->
    Query_result = begin
        _pipe = pog:'query'(
            <<<<"select version::text from pevensie.module_version where module = '"/utf8,
                    Module/binary>>/binary,
                "' limit 1"/utf8>>
        ),
        _pipe@1 = pog:returning(
            _pipe,
            (gleam@dynamic@decode:field(
                0,
                {decoder, fun gleam@dynamic@decode:decode_string/1},
                fun(Datetime_string) ->
                    case tempo@datetime:from_string(Datetime_string) of
                        {ok, Datetime} ->
                            gleam@dynamic@decode:success(Datetime);

                        {error, _} ->
                            gleam@dynamic@decode:failure(
                                tempo@datetime:now_utc(),
                                <<"ModuleVersion"/utf8>>
                            )
                    end
                end
            ))
        ),
        _pipe@2 = pog:execute(_pipe@1, Tx),
        gleam@result:map_error(_pipe@2, fun gleam@string:inspect/1)
    end,
    gleam@result:'try'(
        Query_result,
        fun(Response) -> case erlang:element(3, Response) of
                [] ->
                    {ok, none};

                [Version] ->
                    {ok, {some, Version}};

                _ ->
                    {error, <<"Too many rows"/utf8>>}
            end end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1238).
-spec get_migrations_for_module(binary()) -> {ok, list(binary())} |
    {error, binary()}.
get_migrations_for_module(Module) ->
    gleam@result:'try'(
        begin
            _pipe = gleam_erlang_ffi:priv_directory(
                <<"pevensie_postgres"/utf8>>
            ),
            gleam@result:replace_error(
                _pipe,
                <<"Couldn't get priv directory"/utf8>>
            )
        end,
        fun(Priv) ->
            _assert_subject = begin
                _pipe@1 = [Priv, <<"migrations"/utf8>>, Module],
                gleam@list:reduce(_pipe@1, fun filepath:join/2)
            end,
            {ok, Directory} = case _assert_subject of
                {ok, _} -> _assert_subject;
                _assert_fail ->
                    erlang:error(#{gleam_error => let_assert,
                                message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                                value => _assert_fail,
                                module => <<"pevensie/postgres"/utf8>>,
                                function => <<"get_migrations_for_module"/utf8>>,
                                line => 1244})
            end,
            _pipe@2 = simplifile:get_files(Directory),
            gleam@result:map_error(_pipe@2, fun(Err) -> _pipe@3 = Err,
                    _pipe@4 = gleam@string:inspect(_pipe@3),
                    add_error_context(
                        _pipe@4,
                        <<"Unable to get files in priv directory"/utf8>>
                    ) end)
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1256).
-spec version_from_filename(binary()) -> tempo:date_time().
version_from_filename(Filename) ->
    _assert_subject = begin
        _pipe = Filename,
        _pipe@1 = filepath:split(_pipe),
        gleam@list:last(_pipe@1)
    end,
    {ok, Filename@1} = case _assert_subject of
        {ok, _} -> _assert_subject;
        _assert_fail ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"version_from_filename"/utf8>>,
                        line => 1257})
    end,
    _assert_subject@1 = begin
        _pipe@2 = Filename@1,
        _pipe@3 = gleam@string:split(_pipe@2, <<"_"/utf8>>),
        gleam@list:first(_pipe@3)
    end,
    {ok, Version_string} = case _assert_subject@1 of
        {ok, _} -> _assert_subject@1;
        _assert_fail@1 ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail@1,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"version_from_filename"/utf8>>,
                        line => 1258})
    end,
    _assert_subject@2 = tempo@datetime:parse(
        <<Version_string/binary, "Z"/utf8>>,
        <<"YYYYMMDDHHmmssZ"/utf8>>
    ),
    {ok, Version} = case _assert_subject@2 of
        {ok, _} -> _assert_subject@2;
        _assert_fail@2 ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail@2,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"version_from_filename"/utf8>>,
                        line => 1260})
    end,
    Version.

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1265).
-spec get_migrations_to_apply_for_module(
    binary(),
    gleam@option:option(tempo:date_time())
) -> {ok, gleam@option:option(binary())} | {error, binary()}.
get_migrations_to_apply_for_module(Module, Current_version) ->
    gleam@result:'try'(
        begin
            _pipe = get_migrations_for_module(Module),
            gleam@result:map_error(
                _pipe,
                fun(_capture) ->
                    add_error_context(
                        _capture,
                        <<<<"Failed to get migrations for '"/utf8,
                                Module/binary>>/binary,
                            "' module"/utf8>>
                    )
                end
            )
        end,
        fun(Files) ->
            Files_to_apply = case Current_version of
                none ->
                    Files;

                {some, Current_version@1} ->
                    _pipe@1 = Files,
                    gleam@list:filter(
                        _pipe@1,
                        fun(File) ->
                            tempo@datetime:compare(
                                version_from_filename(File),
                                Current_version@1
                            )
                            =:= gt
                        end
                    )
            end,
            gleam@bool:guard(case Files_to_apply of
                    [] ->
                        true;

                    _ ->
                        false
                end, {ok, none}, fun() ->
                    gleam@result:'try'(
                        begin
                            _pipe@2 = Files_to_apply,
                            _pipe@3 = gleam@list:try_map(
                                _pipe@2,
                                fun simplifile:read/1
                            ),
                            _pipe@4 = gleam@result:map(
                                _pipe@3,
                                fun(_capture@1) ->
                                    gleam@string:join(_capture@1, <<"\n"/utf8>>)
                                end
                            ),
                            gleam@result:map_error(
                                _pipe@4,
                                fun(Err) ->
                                    add_error_context(
                                        gleam@string:inspect(Err),
                                        <<"Failed to read migrations"/utf8>>
                                    )
                                end
                            )
                        end,
                        fun(Migration_sql) ->
                            _assert_subject = gleam@list:last(Files_to_apply),
                            {ok, Latest_migration} = case _assert_subject of
                                {ok, _} -> _assert_subject;
                                _assert_fail ->
                                    erlang:error(#{gleam_error => let_assert,
                                                message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                                                value => _assert_fail,
                                                module => <<"pevensie/postgres"/utf8>>,
                                                function => <<"get_migrations_to_apply_for_module"/utf8>>,
                                                line => 1305})
                            end,
                            New_version = version_from_filename(
                                Latest_migration
                            ),
                            New_version_sql = <<<<<<<<"insert into pevensie.module_version (module, version)
values ('"/utf8,
                                            Module/binary>>/binary,
                                        "', timestamptz '"/utf8>>/binary,
                                    (begin
                                        _pipe@5 = New_version,
                                        tempo@datetime:to_string(_pipe@5)
                                    end)/binary>>/binary,
                                "')
on conflict (module)
do update set version = EXCLUDED.version;\n"/utf8>>,
                            {ok,
                                {some,
                                    <<<<Migration_sql/binary, "\n"/utf8>>/binary,
                                        New_version_sql/binary>>}}
                        end
                    )
                end)
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1318).
-spec apply_migrations_for_module(pog:connection(), binary(), binary()) -> {ok,
        nil} |
    {error, binary()}.
apply_migrations_for_module(Tx, Module, Migration_sql) ->
    gleam_stdlib:println_error(
        <<<<"Applying migrations for module '"/utf8, Module/binary>>/binary,
            "'"/utf8>>
    ),
    gleam@result:'try'(
        begin
            _pipe@1 = gleam@string:last(
                begin
                    _pipe = Migration_sql,
                    gleam@string:trim_end(_pipe)
                end
            ),
            gleam@result:replace_error(_pipe@1, <<"Empty migration file"/utf8>>)
        end,
        fun(Last_char) ->
            Sql = case Last_char of
                <<";"/utf8>> ->
                    Migration_sql;

                _ ->
                    <<Migration_sql/binary, ";"/utf8>>
            end,
            Sql@1 = <<<<"do $pevensiemigration$ begin"/utf8, Sql/binary>>/binary,
                " end;$pevensiemigration$"/utf8>>,
            Query_result = begin
                _pipe@2 = pog:'query'(Sql@1),
                _pipe@3 = pog:execute(_pipe@2, Tx),
                gleam@result:map_error(_pipe@3, fun gleam@string:inspect/1)
            end,
            gleam@result:'try'(
                Query_result,
                fun(_) ->
                    gleam_stdlib:println_error(<<"ok."/utf8>>),
                    {ok, nil}
                end
            )
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1348).
-spec handle_module_migration(pog:connection(), binary(), boolean()) -> {ok,
        nil} |
    {error, binary()}.
handle_module_migration(Tx, Module, Apply) ->
    gleam_stdlib:print_error(<<"Checking schema... "/utf8>>),
    gleam@result:'try'(
        begin
            _pipe = check_pevensie_schema_exists(Tx),
            gleam@result:map_error(
                _pipe,
                fun(_capture) ->
                    add_error_context(
                        _capture,
                        <<"Unable to check if 'pevensie' schema exists"/utf8>>
                    )
                end
            )
        end,
        fun(Schema_exists) ->
            gleam_stdlib:println_error(<<"ok."/utf8>>),
            gleam_stdlib:print_error(
                <<<<"Checking current version of '"/utf8, Module/binary>>/binary,
                    "' module... "/utf8>>
            ),
            Version_result = case Schema_exists of
                false ->
                    {ok, none};

                true ->
                    _pipe@1 = get_module_version(Tx, Module),
                    gleam@result:map_error(
                        _pipe@1,
                        fun(_capture@1) ->
                            add_error_context(
                                _capture@1,
                                <<<<"Unable to check '"/utf8, Module/binary>>/binary,
                                    "' module version"/utf8>>
                            )
                        end
                    )
            end,
            gleam@result:'try'(
                Version_result,
                fun(Current_version) ->
                    gleam_stdlib:println_error(
                        <<"ok. Current version: "/utf8,
                            (begin
                                _pipe@2 = Current_version,
                                _pipe@3 = gleam@option:map(
                                    _pipe@2,
                                    fun tempo@datetime:to_string/1
                                ),
                                gleam@option:unwrap(_pipe@3, <<"none"/utf8>>)
                            end)/binary>>
                    ),
                    gleam_stdlib:print_error(
                        <<<<"Getting migrations for module '"/utf8,
                                Module/binary>>/binary,
                            "'... "/utf8>>
                    ),
                    gleam@result:'try'(
                        begin
                            _pipe@4 = get_migrations_to_apply_for_module(
                                Module,
                                Current_version
                            ),
                            gleam@result:map_error(
                                _pipe@4,
                                fun(_capture@2) ->
                                    add_error_context(
                                        _capture@2,
                                        <<<<"Failed to get migrations to apply for '"/utf8,
                                                Module/binary>>/binary,
                                            "'"/utf8>>
                                    )
                                end
                            )
                        end,
                        fun(Migration_sql) ->
                            gleam_stdlib:println_error(<<"ok."/utf8>>),
                            case Migration_sql of
                                none ->
                                    gleam_stdlib:println_error(
                                        <<<<"No migrations to apply for module '"/utf8,
                                                Module/binary>>/binary,
                                            "'\n"/utf8>>
                                    ),
                                    {ok, nil};

                                {some, Migration_sql@1} ->
                                    case Apply of
                                        true ->
                                            _pipe@5 = apply_migrations_for_module(
                                                Tx,
                                                Module,
                                                Migration_sql@1
                                            ),
                                            gleam@result:map_error(
                                                _pipe@5,
                                                fun(_capture@3) ->
                                                    add_error_context(
                                                        _capture@3,
                                                        <<<<"Failed to apply migrations for '"/utf8,
                                                                Module/binary>>/binary,
                                                            "'"/utf8>>
                                                    )
                                                end
                                            );

                                        false ->
                                            gleam_stdlib:println(
                                                Migration_sql@1
                                            ),
                                            {ok, nil}
                                    end
                            end
                        end
                    )
                end
            )
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1414).
-spec connection_string_flag() -> glint:flag(binary()).
connection_string_flag() ->
    _pipe = glint:string_flag(<<"addr"/utf8>>),
    _pipe@1 = glint:flag_help(
        _pipe,
        <<"The connection string for your Postgres database"/utf8>>
    ),
    glint:flag_default(
        _pipe@1,
        <<"postgresql://postgres:postgres@localhost:5432/postgres"/utf8>>
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1422).
-spec apply_flag() -> glint:flag(boolean()).
apply_flag() ->
    _pipe = glint:bool_flag(<<"apply"/utf8>>),
    _pipe@1 = glint:flag_help(
        _pipe,
        <<"Apply migrations instead of just printing them"/utf8>>
    ),
    glint:flag_default(_pipe@1, false).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1428).
-spec migrate_command() -> glint:command({ok, nil} | {error, snag:snag()}).
migrate_command() ->
    glint:command_help(
        <<"Migrate Pevensie modules in your Postgres database"/utf8>>,
        fun() ->
            glint:flag(
                connection_string_flag(),
                fun(Connection_string_arg) ->
                    glint:flag(
                        apply_flag(),
                        fun(Apply_flag) ->
                            glint:unnamed_args(
                                {min_args, 1},
                                fun() ->
                                    glint:command(
                                        fun(_, Unnamed, Flags) ->
                                            gleam@result:'try'(
                                                Connection_string_arg(Flags),
                                                fun(Connection_string) ->
                                                    gleam@result:'try'(
                                                        Apply_flag(Flags),
                                                        fun(Apply) ->
                                                            Deduped_modules = begin
                                                                _pipe = Unnamed,
                                                                _pipe@1 = gleam@list:filter(
                                                                    _pipe,
                                                                    fun(Module) ->
                                                                        (Module
                                                                        =:= <<"auth"/utf8>>)
                                                                        orelse (Module
                                                                        =:= <<"cache"/utf8>>)
                                                                    end
                                                                ),
                                                                _pipe@2 = gleam@set:from_list(
                                                                    _pipe@1
                                                                ),
                                                                gleam@set:to_list(
                                                                    _pipe@2
                                                                )
                                                            end,
                                                            gleam@result:'try'(
                                                                case Deduped_modules of
                                                                    [] ->
                                                                        {error,
                                                                            {snag,
                                                                                <<"No valid modules specified. Valid modules: auth, cache"/utf8>>,
                                                                                [<<"no modules"/utf8>>]}};

                                                                    Modules ->
                                                                        {ok,
                                                                            Modules}
                                                                end,
                                                                fun(Modules@1) ->
                                                                    gleam@result:'try'(
                                                                        begin
                                                                            _pipe@3 = pog:url_config(
                                                                                Connection_string
                                                                            ),
                                                                            gleam@result:replace_error(
                                                                                _pipe@3,
                                                                                {snag,
                                                                                    <<"Invalid Postgres connection string"/utf8>>,
                                                                                    [<<"invalid connection string"/utf8>>]}
                                                                            )
                                                                        end,
                                                                        fun(
                                                                            Config
                                                                        ) ->
                                                                            Conn = pog_ffi:connect(
                                                                                Config
                                                                            ),
                                                                            Transaction_result = pog_ffi:transaction(
                                                                                Conn,
                                                                                fun(
                                                                                    Tx
                                                                                ) ->
                                                                                    gleam@result:'try'(
                                                                                        handle_module_migration(
                                                                                            Tx,
                                                                                            <<"base"/utf8>>,
                                                                                            Apply
                                                                                        ),
                                                                                        fun(
                                                                                            _
                                                                                        ) ->
                                                                                            Migration_result = begin
                                                                                                _pipe@4 = Modules@1,
                                                                                                gleam@list:try_each(
                                                                                                    _pipe@4,
                                                                                                    fun(
                                                                                                        _capture
                                                                                                    ) ->
                                                                                                        handle_module_migration(
                                                                                                            Tx,
                                                                                                            _capture,
                                                                                                            Apply
                                                                                                        )
                                                                                                    end
                                                                                                )
                                                                                            end,
                                                                                            gleam@result:'try'(
                                                                                                Migration_result,
                                                                                                fun(
                                                                                                    _
                                                                                                ) ->
                                                                                                    case Apply of
                                                                                                        true ->
                                                                                                            gleam_stdlib:println_error(
                                                                                                                <<"\nSuccess! Applied migrations for modules: "/utf8,
                                                                                                                    (begin
                                                                                                                        _pipe@5 = Modules@1,
                                                                                                                        gleam@string:join(
                                                                                                                            _pipe@5,
                                                                                                                            <<","/utf8>>
                                                                                                                        )
                                                                                                                    end)/binary>>
                                                                                                            );

                                                                                                        false ->
                                                                                                            nil
                                                                                                    end,
                                                                                                    {ok,
                                                                                                        nil}
                                                                                                end
                                                                                            )
                                                                                        end
                                                                                    )
                                                                                end
                                                                            ),
                                                                            case Transaction_result of
                                                                                {ok,
                                                                                    _} ->
                                                                                    {ok,
                                                                                        nil};

                                                                                {error,
                                                                                    {transaction_rolled_back,
                                                                                        Msg}} ->
                                                                                    {error,
                                                                                        {snag,
                                                                                            Msg,
                                                                                            [<<"transaction rolled back"/utf8>>]}};

                                                                                {error,
                                                                                    {transaction_query_error,
                                                                                        Err}} ->
                                                                                    {error,
                                                                                        {snag,
                                                                                            <<"Query error: "/utf8,
                                                                                                (gleam@string:inspect(
                                                                                                    Err
                                                                                                ))/binary>>,
                                                                                            [<<"query error"/utf8>>]}}
                                                                            end
                                                                        end
                                                                    )
                                                                end
                                                            )
                                                        end
                                                    )
                                                end
                                            )
                                        end
                                    )
                                end
                            )
                        end
                    )
                end
            )
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 1500).
-spec main() -> nil.
main() ->
    Cli = begin
        _pipe = glint:new(),
        _pipe@1 = glint:with_name(_pipe, <<"pevensie/drivers/postgres"/utf8>>),
        _pipe@2 = glint:as_module(_pipe@1),
        _pipe@3 = glint:pretty_help(_pipe@2, glint:default_pretty_help()),
        glint:add(_pipe@3, [<<"migrate"/utf8>>], migrate_command())
    end,
    glint:run_and_handle(
        Cli,
        erlang:element(4, argv:load()),
        fun(Cli_result) ->
            gleam@result:map_error(
                Cli_result,
                fun(Errors) ->
                    gleam_stdlib:println_error(
                        <<"Command failed with an error: "/utf8,
                            (erlang:element(2, Errors))/binary>>
                    )
                end
            )
        end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 374).
-spec list_users(
    postgres(),
    integer(),
    integer(),
    pevensie@auth:user_search_fields(),
    gleam@dynamic@decode:decoder(QEI)
) -> {ok, list(pevensie@auth:user(QEI))} |
    {error, pevensie@auth:get_error(postgres_error())}.
list_users(Driver, Limit, Offset, Filters, User_metadata_decoder) ->
    {postgres, _, {some, Conn}} = case Driver of
        {postgres, _, {some, _}} -> Driver;
        _assert_fail ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"list_users"/utf8>>,
                        line => 381})
    end,
    Filter_fields = begin
        _pipe = [{<<"id"/utf8>>, erlang:element(2, Filters)},
            {<<"email"/utf8>>, erlang:element(3, Filters)},
            {<<"phone_number"/utf8>>, erlang:element(4, Filters)}],
        _pipe@1 = gleam@list:filter(
            _pipe,
            fun(Field) -> gleam@option:is_some(erlang:element(2, Field)) end
        ),
        gleam@list:index_map(
            _pipe@1,
            fun(Field@1, Index) ->
                {<<<<<<(erlang:element(1, Field@1))/binary,
                                "::text like any($"/utf8>>/binary,
                            (erlang:integer_to_binary(Index + 1))/binary>>/binary,
                        ")"/utf8>>,
                    begin
                        _pipe@2 = erlang:element(2, Field@1),
                        _pipe@3 = gleam@option:unwrap(_pipe@2, []),
                        pog:array(fun pog_ffi:coerce/1, _pipe@3)
                    end}
            end
        )
    end,
    Filter_sql = begin
        _pipe@4 = Filter_fields,
        _pipe@5 = gleam@list:map(_pipe@4, fun gleam@pair:first/1),
        gleam@string:join(_pipe@5, <<" or "/utf8>>)
    end,
    Sql = <<<<<<<<<<<<<<"
    select
      "/utf8,
                                "
  id::text,
  -- Convert timestamp fields to UNIX epoch microseconds
  (extract(epoch from created_at) * 1000000)::bigint as created_at,
  (extract(epoch from updated_at) * 1000000)::bigint as updated_at,
  (extract(epoch from deleted_at) * 1000000)::bigint as deleted_at,
  role,
  email,
  password_hash,
  (extract(epoch from email_confirmed_at) * 1000000)::bigint as email_confirmed_at,
  phone_number,
  (extract(epoch from phone_number_confirmed_at) * 1000000)::bigint as phone_number_confirmed_at,
  (extract(epoch from last_sign_in) * 1000000)::bigint as last_sign_in,
  app_metadata,
  user_metadata,
  (extract(epoch from banned_until) * 1000000)::bigint as banned_until
"/utf8>>/binary,
                            "
    from pevensie.\"user\"
    where "/utf8>>/binary,
                        Filter_sql/binary>>/binary,
                    " and deleted_at is null
    limit "/utf8>>/binary,
                (erlang:integer_to_binary(Limit))/binary>>/binary,
            "
    offset "/utf8>>/binary,
        (erlang:integer_to_binary(Offset))/binary>>,
    _pipe@6 = Filter_fields,
    _pipe@8 = gleam@list:fold(
        _pipe@6,
        pog:'query'(Sql),
        fun(Query, Field@2) -> _pipe@7 = Query,
            pog:parameter(_pipe@7, erlang:element(2, Field@2)) end
    ),
    _pipe@9 = pog:returning(_pipe@8, user_decoder(User_metadata_decoder)),
    _pipe@10 = pog:execute(_pipe@9, Conn),
    _pipe@11 = gleam@result:map(
        _pipe@10,
        fun(Response) -> erlang:element(3, Response) end
    ),
    gleam@result:map_error(_pipe@11, fun(Err) -> _pipe@12 = Err,
            _pipe@13 = pog_query_error_to_postgres_error(_pipe@12),
            {get_driver_error, _pipe@13} end).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 427).
-spec create_user(
    postgres(),
    pevensie@auth:user_create(QEP),
    gleam@dynamic@decode:decoder(QEP),
    fun((QEP) -> gleam@json:json())
) -> {ok, pevensie@auth:user(QEP)} |
    {error, pevensie@auth:create_error(postgres_error())}.
create_user(Driver, User, User_metadata_decoder, User_metadata_encoder) ->
    {postgres, _, {some, Conn}} = case Driver of
        {postgres, _, {some, _}} -> Driver;
        _assert_fail ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"create_user"/utf8>>,
                        line => 433})
    end,
    Sql = <<"
    insert into pevensie.\"user\" (
      role,
      email,
      password_hash,
      email_confirmed_at,
      phone_number,
      phone_number_confirmed_at,
      app_metadata,
      user_metadata
    ) values (
      $1,
      $2,
      $3,
      $4::timestamptz,
      $5,
      $6::timestamptz,
      $7::jsonb,
      $8::jsonb
    )
    returning
      "/utf8,
        "
  id::text,
  -- Convert timestamp fields to UNIX epoch microseconds
  (extract(epoch from created_at) * 1000000)::bigint as created_at,
  (extract(epoch from updated_at) * 1000000)::bigint as updated_at,
  (extract(epoch from deleted_at) * 1000000)::bigint as deleted_at,
  role,
  email,
  password_hash,
  (extract(epoch from email_confirmed_at) * 1000000)::bigint as email_confirmed_at,
  phone_number,
  (extract(epoch from phone_number_confirmed_at) * 1000000)::bigint as phone_number_confirmed_at,
  (extract(epoch from last_sign_in) * 1000000)::bigint as last_sign_in,
  app_metadata,
  user_metadata,
  (extract(epoch from banned_until) * 1000000)::bigint as banned_until
"/utf8>>,
    Query_result = begin
        _pipe = pog:'query'(Sql),
        _pipe@1 = pog:parameter(
            _pipe,
            pog:nullable(fun pog_ffi:coerce/1, erlang:element(2, User))
        ),
        _pipe@2 = pog:parameter(
            _pipe@1,
            pog_ffi:coerce(erlang:element(3, User))
        ),
        _pipe@3 = pog:parameter(
            _pipe@2,
            pog:nullable(fun pog_ffi:coerce/1, erlang:element(4, User))
        ),
        _pipe@5 = pog:parameter(
            _pipe@3,
            pog:nullable(
                fun pog:timestamp/1,
                begin
                    _pipe@4 = erlang:element(5, User),
                    gleam@option:map(
                        _pipe@4,
                        fun tempo_datetime_to_pog_timestamp/1
                    )
                end
            )
        ),
        _pipe@6 = pog:parameter(
            _pipe@5,
            pog:nullable(fun pog_ffi:coerce/1, erlang:element(6, User))
        ),
        _pipe@8 = pog:parameter(
            _pipe@6,
            pog:nullable(
                fun pog:timestamp/1,
                begin
                    _pipe@7 = erlang:element(7, User),
                    gleam@option:map(
                        _pipe@7,
                        fun tempo_datetime_to_pog_timestamp/1
                    )
                end
            )
        ),
        _pipe@10 = pog:parameter(
            _pipe@8,
            pog_ffi:coerce(
                begin
                    _pipe@9 = pevensie@auth:app_metadata_encoder(
                        erlang:element(9, User)
                    ),
                    gleam@json:to_string(_pipe@9)
                end
            )
        ),
        _pipe@12 = pog:parameter(
            _pipe@10,
            pog_ffi:coerce(
                begin
                    _pipe@11 = User_metadata_encoder(erlang:element(10, User)),
                    gleam@json:to_string(_pipe@11)
                end
            )
        ),
        _pipe@13 = pog:returning(_pipe@12, user_decoder(User_metadata_decoder)),
        _pipe@14 = pog:execute(_pipe@13, Conn),
        gleam@result:map_error(
            _pipe@14,
            fun(_capture) ->
                pog_query_error_to_pevensie_error(
                    _capture,
                    fun(Field@0) -> {create_driver_error, Field@0} end
                )
            end
        )
    end,
    gleam@result:'try'(
        Query_result,
        fun(Response) -> case erlang:element(3, Response) of
                [User@1] ->
                    {ok, User@1};

                [] ->
                    {error, created_too_few_records};

                [_ | _] ->
                    {error, created_too_many_records}
            end end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 504).
-spec update_user(
    postgres(),
    binary(),
    binary(),
    pevensie@auth:user_update(QFA),
    gleam@dynamic@decode:decoder(QFA),
    fun((QFA) -> gleam@json:json())
) -> {ok, pevensie@auth:user(QFA)} |
    {error, pevensie@auth:update_error(postgres_error())}.
update_user(
    Driver,
    Field,
    Value,
    User,
    User_metadata_decoder,
    User_metadata_encoder
) ->
    {postgres, _, {some, Conn}} = case Driver of
        {postgres, _, {some, _}} -> Driver;
        _assert_fail ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"update_user"/utf8>>,
                        line => 512})
    end,
    Optional_timestamp_to_pog = fun(Timestamp) -> _pipe = Timestamp,
        _pipe@1 = gleam@option:map(_pipe, fun tempo_datetime_to_pog_timestamp/1),
        pog:nullable(fun pog:timestamp/1, _pipe@1) end,
    Record_to_pog = fun(Record, Encoder) ->
        pog_ffi:coerce(
            begin
                _pipe@2 = Encoder(Record),
                gleam@json:to_string(_pipe@2)
            end
        )
    end,
    Fields = [{<<"role"/utf8>>,
            update_field_to_sql(
                erlang:element(2, User),
                fun(_capture) ->
                    pog:nullable(fun pog_ffi:coerce/1, _capture)
                end
            )},
        {<<"email"/utf8>>,
            update_field_to_sql(erlang:element(3, User), fun pog_ffi:coerce/1)},
        {<<"password_hash"/utf8>>,
            update_field_to_sql(
                erlang:element(4, User),
                fun(_capture@1) ->
                    pog:nullable(fun pog_ffi:coerce/1, _capture@1)
                end
            )},
        {<<"email_confirmed_at"/utf8>>,
            update_field_to_sql(
                erlang:element(5, User),
                Optional_timestamp_to_pog
            )},
        {<<"phone_number"/utf8>>,
            update_field_to_sql(
                erlang:element(6, User),
                fun(_capture@2) ->
                    pog:nullable(fun pog_ffi:coerce/1, _capture@2)
                end
            )},
        {<<"phone_number_confirmed_at"/utf8>>,
            update_field_to_sql(
                erlang:element(7, User),
                Optional_timestamp_to_pog
            )},
        {<<"last_sign_in"/utf8>>,
            update_field_to_sql(
                erlang:element(8, User),
                Optional_timestamp_to_pog
            )},
        {<<"app_metadata"/utf8>>,
            update_field_to_sql(
                erlang:element(9, User),
                fun(_capture@3) ->
                    Record_to_pog(
                        _capture@3,
                        fun pevensie@auth:app_metadata_encoder/1
                    )
                end
            )},
        {<<"user_metadata"/utf8>>,
            update_field_to_sql(
                erlang:element(10, User),
                fun(_capture@4) ->
                    Record_to_pog(_capture@4, User_metadata_encoder)
                end
            )}],
    Fields_to_update = begin
        _pipe@3 = Fields,
        gleam@list:filter_map(
            _pipe@3,
            fun(Field@1) -> case erlang:element(2, Field@1) of
                    {set, Value@1} ->
                        {ok, {erlang:element(1, Field@1), Value@1}};

                    ignore ->
                        {error, nil}
                end end
        )
    end,
    Field_setters = begin
        _pipe@4 = Fields_to_update,
        _pipe@5 = gleam@list:index_map(
            _pipe@4,
            fun(Field@2, Index) ->
                <<<<(erlang:element(1, Field@2))/binary, " = $"/utf8>>/binary,
                    (erlang:integer_to_binary(Index + 1))/binary>>
            end
        ),
        gleam@string:join(_pipe@5, <<", "/utf8>>)
    end,
    Field_setters@1 = case Field_setters of
        <<""/utf8>> ->
            <<"updated_at = now()"/utf8>>;

        _ ->
            <<Field_setters/binary, ", updated_at = now()"/utf8>>
    end,
    Update_values = begin
        _pipe@6 = Fields_to_update,
        gleam@list:map(_pipe@6, fun gleam@pair:second/1)
    end,
    Sql = <<<<<<<<<<<<<<"
    update pevensie.\"user\"
    set "/utf8,
                                Field_setters@1/binary>>/binary,
                            "
    where "/utf8>>/binary,
                        Field/binary>>/binary,
                    " = $"/utf8>>/binary,
                (erlang:integer_to_binary(erlang:length(Fields_to_update) + 1))/binary>>/binary,
            " and deleted_at is null
    returning "/utf8>>/binary,
        "
  id::text,
  -- Convert timestamp fields to UNIX epoch microseconds
  (extract(epoch from created_at) * 1000000)::bigint as created_at,
  (extract(epoch from updated_at) * 1000000)::bigint as updated_at,
  (extract(epoch from deleted_at) * 1000000)::bigint as deleted_at,
  role,
  email,
  password_hash,
  (extract(epoch from email_confirmed_at) * 1000000)::bigint as email_confirmed_at,
  phone_number,
  (extract(epoch from phone_number_confirmed_at) * 1000000)::bigint as phone_number_confirmed_at,
  (extract(epoch from last_sign_in) * 1000000)::bigint as last_sign_in,
  app_metadata,
  user_metadata,
  (extract(epoch from banned_until) * 1000000)::bigint as banned_until
"/utf8>>,
    Query_result = begin
        _pipe@7 = Update_values,
        _pipe@9 = gleam@list:fold(
            _pipe@7,
            pog:'query'(Sql),
            fun(Query, Value@2) -> _pipe@8 = Query,
                pog:parameter(_pipe@8, Value@2) end
        ),
        _pipe@10 = pog:parameter(_pipe@9, pog_ffi:coerce(Value)),
        _pipe@11 = pog:returning(_pipe@10, user_decoder(User_metadata_decoder)),
        _pipe@12 = pog:execute(_pipe@11, Conn),
        gleam@result:map_error(
            _pipe@12,
            fun(_capture@5) ->
                pog_query_error_to_pevensie_error(
                    _capture@5,
                    fun(Field@0) -> {update_driver_error, Field@0} end
                )
            end
        )
    end,
    gleam@result:'try'(
        Query_result,
        fun(Response) -> case erlang:element(3, Response) of
                [User@1] ->
                    {ok, User@1};

                [] ->
                    {error, updated_too_few_records};

                _ ->
                    {error, updated_too_many_records}
            end end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 624).
-spec delete_user(
    postgres(),
    binary(),
    binary(),
    gleam@dynamic@decode:decoder(QFI)
) -> {ok, pevensie@auth:user(QFI)} |
    {error, pevensie@auth:delete_error(postgres_error())}.
delete_user(Driver, Field, Value, User_metadata_decoder) ->
    {postgres, _, {some, Conn}} = case Driver of
        {postgres, _, {some, _}} -> Driver;
        _assert_fail ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"delete_user"/utf8>>,
                        line => 630})
    end,
    Sql = <<<<<<"
    update pevensie.\"user\"
    set deleted_at = now()
    where "/utf8,
                Field/binary>>/binary,
            " = $1 and deleted_at is null
    returning "/utf8>>/binary,
        "
  id::text,
  -- Convert timestamp fields to UNIX epoch microseconds
  (extract(epoch from created_at) * 1000000)::bigint as created_at,
  (extract(epoch from updated_at) * 1000000)::bigint as updated_at,
  (extract(epoch from deleted_at) * 1000000)::bigint as deleted_at,
  role,
  email,
  password_hash,
  (extract(epoch from email_confirmed_at) * 1000000)::bigint as email_confirmed_at,
  phone_number,
  (extract(epoch from phone_number_confirmed_at) * 1000000)::bigint as phone_number_confirmed_at,
  (extract(epoch from last_sign_in) * 1000000)::bigint as last_sign_in,
  app_metadata,
  user_metadata,
  (extract(epoch from banned_until) * 1000000)::bigint as banned_until
"/utf8>>,
    Query_result = begin
        _pipe = pog:'query'(Sql),
        _pipe@1 = pog:parameter(_pipe, pog_ffi:coerce(Value)),
        _pipe@2 = pog:returning(_pipe@1, user_decoder(User_metadata_decoder)),
        _pipe@3 = pog:execute(_pipe@2, Conn),
        gleam@result:map_error(
            _pipe@3,
            fun(_capture) ->
                pog_query_error_to_pevensie_error(
                    _capture,
                    fun(Field@0) -> {delete_driver_error, Field@0} end
                )
            end
        )
    end,
    gleam@result:'try'(
        Query_result,
        fun(Response) -> case erlang:element(3, Response) of
                [User] ->
                    {ok, User};

                [] ->
                    {error, deleted_too_few_records};

                _ ->
                    {error, deleted_too_many_records}
            end end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 689).
-spec get_session(
    postgres(),
    binary(),
    gleam@option:option(pevensie@net:ip_address()),
    gleam@option:option(binary())
) -> {ok, pevensie@auth:session()} |
    {error, pevensie@auth:get_error(postgres_error())}.
get_session(Driver, Session_id, Ip, User_agent) ->
    {postgres, _, {some, Conn}} = case Driver of
        {postgres, _, {some, _}} -> Driver;
        _assert_fail ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"get_session"/utf8>>,
                        line => 695})
    end,
    Sql = <<<<"
    select
      "/utf8,
            "
  id::text,
  user_id::text,
  (extract(epoch from created_at) * 1000000)::bigint as created_at,
  (extract(epoch from expires_at) * 1000000)::bigint as expires_at,
  host(ip)::text,
  user_agent
"/utf8>>/binary,
        ",
      (expires_at is not null and expires_at < now()) as expired
    from pevensie.\"session\"
    where id = $1
    "/utf8>>,
    Additional_fields = [case Ip of
            none ->
                {<<"ip is null"/utf8>>, none};

            {some, _} ->
                {<<"ip = $"/utf8>>,
                    {some,
                        pog:nullable(
                            fun pog_ffi:coerce/1,
                            begin
                                _pipe = Ip,
                                gleam@option:map(
                                    _pipe,
                                    fun pevensie@net:format_ip_address/1
                                )
                            end
                        )}}
        end, case User_agent of
            none ->
                {<<"user_agent is null"/utf8>>, none};

            {some, _} ->
                {<<"user_agent = $"/utf8>>,
                    {some, pog:nullable(fun pog_ffi:coerce/1, User_agent)}}
        end],
    {Sql@2, _} = gleam@list:fold(
        Additional_fields,
        {Sql, 2},
        fun(Sql_and_counter, Field) ->
            {Sql@1, Counter} = Sql_and_counter,
            case Field of
                {Stmt, none} ->
                    {<<<<Sql@1/binary, " and "/utf8>>/binary, Stmt/binary>>,
                        Counter};

                {Stmt@1, {some, _}} ->
                    {<<<<<<Sql@1/binary, " and "/utf8>>/binary, Stmt@1/binary>>/binary,
                            (erlang:integer_to_binary(Counter))/binary>>,
                        Counter + 1}
            end
        end
    ),
    Query = begin
        _pipe@1 = pog:'query'(Sql@2),
        pog:parameter(_pipe@1, pog_ffi:coerce(Session_id))
    end,
    Query@2 = begin
        _pipe@2 = Additional_fields,
        gleam@list:fold(_pipe@2, Query, fun(Query@1, Field@1) -> case Field@1 of
                    {_, {some, Param}} ->
                        _pipe@3 = Query@1,
                        pog:parameter(_pipe@3, Param);

                    _ ->
                        Query@1
                end end)
    end,
    Query_result = begin
        _pipe@4 = Query@2,
        _pipe@5 = pog:returning(
            _pipe@4,
            (gleam@dynamic@decode:then(
                session_decoder(),
                fun(Session) ->
                    gleam@dynamic@decode:field(
                        6,
                        {decoder, fun gleam@dynamic@decode:decode_bool/1},
                        fun(Expired) ->
                            gleam@dynamic@decode:success({Session, Expired})
                        end
                    )
                end
            ))
        ),
        _pipe@6 = pog:execute(_pipe@5, Conn),
        gleam@result:map_error(
            _pipe@6,
            fun(_capture) ->
                pog_query_error_to_pevensie_error(
                    _capture,
                    fun(Field@0) -> {get_driver_error, Field@0} end
                )
            end
        )
    end,
    gleam@result:'try'(
        Query_result,
        fun(Response) -> case erlang:element(3, Response) of
                [{Session@1, false}] ->
                    {ok, Session@1};

                [{_, true}] ->
                    gleam@erlang@process:start(
                        fun() -> delete_session(Driver, Session_id) end,
                        false
                    ),
                    {error, got_too_few_records};

                [] ->
                    {error, got_too_few_records};

                _ ->
                    {error, got_too_many_records}
            end end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 803).
-spec create_session(
    postgres(),
    binary(),
    gleam@option:option(pevensie@net:ip_address()),
    gleam@option:option(binary()),
    gleam@option:option(integer())
) -> {ok, pevensie@auth:session()} |
    {error, pevensie@auth:create_error(postgres_error())}.
create_session(Driver, User_id, Ip, User_agent, Ttl_seconds) ->
    {postgres, _, {some, Conn}} = case Driver of
        {postgres, _, {some, _}} -> Driver;
        _assert_fail ->
            erlang:error(#{gleam_error => let_assert,
                        message => <<"Pattern match failed, no pattern matched the value."/utf8>>,
                        value => _assert_fail,
                        module => <<"pevensie/postgres"/utf8>>,
                        function => <<"create_session"/utf8>>,
                        line => 810})
    end,
    Expires_at_sql = case Ttl_seconds of
        none ->
            <<"null"/utf8>>;

        {some, Ttl_seconds@1} ->
            <<<<"now() + interval '"/utf8,
                    (erlang:integer_to_binary(Ttl_seconds@1))/binary>>/binary,
                " seconds'"/utf8>>
    end,
    Ip_string = case Ip of
        none ->
            <<"null"/utf8>>;

        {some, Ip@1} ->
            <<<<"'"/utf8, (pevensie@net:format_ip_address(Ip@1))/binary>>/binary,
                "'::inet"/utf8>>
    end,
    _ = pevensie@net:parse_ip_address(<<"127.0.0.1"/utf8>>),
    Sql = <<<<<<<<<<"
    insert into pevensie.\"session\" (
      user_id,
      ip,
      user_agent,
      expires_at
    ) values (
      $1,
      "/utf8,
                        Ip_string/binary>>/binary,
                    ",
      $2,
      "/utf8>>/binary,
                Expires_at_sql/binary>>/binary,
            "
    )
    returning
      "/utf8>>/binary,
        "
  id::text,
  user_id::text,
  (extract(epoch from created_at) * 1000000)::bigint as created_at,
  (extract(epoch from expires_at) * 1000000)::bigint as expires_at,
  host(ip)::text,
  user_agent
"/utf8>>,
    Query_result = begin
        _pipe = pog:'query'(Sql),
        _pipe@1 = pog:parameter(_pipe, pog_ffi:coerce(User_id)),
        _pipe@2 = pog:parameter(
            _pipe@1,
            pog:nullable(fun pog_ffi:coerce/1, User_agent)
        ),
        _pipe@3 = pog:returning(_pipe@2, session_decoder()),
        _pipe@4 = pog:execute(_pipe@3, Conn),
        gleam@result:map_error(
            _pipe@4,
            fun(_capture) ->
                pog_query_error_to_pevensie_error(
                    _capture,
                    fun(Field@0) -> {create_driver_error, Field@0} end
                )
            end
        )
    end,
    gleam@result:'try'(
        Query_result,
        fun(Response) -> case erlang:element(3, Response) of
                [] ->
                    {error, created_too_few_records};

                [Session] ->
                    {ok, Session};

                _ ->
                    {error, created_too_many_records}
            end end
    ).

-file("/Users/isaac/repos/pevensie_postgres/src/pevensie/postgres.gleam", 223).
-spec new_auth_driver(postgres_config()) -> pevensie@auth:auth_driver(postgres(), postgres_error(), any()).
new_auth_driver(Config) ->
    {auth_driver,
        {postgres, Config, none},
        fun connect/1,
        fun disconnect/1,
        fun list_users/5,
        fun create_user/4,
        fun update_user/6,
        fun delete_user/4,
        fun get_session/4,
        fun create_session/5,
        fun delete_session/2,
        fun create_one_time_token/4,
        fun validate_one_time_token/4,
        fun use_one_time_token/4,
        fun delete_one_time_token/4}.
